@@ -1,44 +1,43 @@
 import asyncio
 import collections
-from functools import lru_cache, partial
 import importlib
 import logging
-import re
 import os
 import pprint
+import re
 import secrets
 import urllib.parse
-
-from fastapi import FastAPI, APIRouter, Request, Response
-from fastapi.openapi.utils import get_openapi
-from fastapi.middleware.cors import CORSMiddleware
+from functools import lru_cache, partial
 
 from bluesky_queueserver.manager.comms import validate_zmq_key
 from bluesky_queueserver_api.zmq.aio import REManagerAPI
+from fastapi import APIRouter, FastAPI, Request, Response
+from fastapi.middleware.cors import CORSMiddleware
+from fastapi.openapi.utils import get_openapi
 
+from .authentication import Mode
 from .console_output import CollectPublishedConsoleOutput
+from .core import PatchedStreamingResponse
+from .database.core import purge_expired
 from .resources import SERVER_RESOURCES as SR
-from .utils import (
-    get_default_login_data,
-    get_authenticators,
-    get_api_access_manager,
-    get_resource_access_manager,
-    record_timing,
-    API_KEY_COOKIE_NAME,
-    CSRF_COOKIE_NAME,
-)
-
 from .routers import core_api
 from .settings import get_settings
-from .authentication import Mode
-from .database.core import purge_expired
-from .core import PatchedStreamingResponse
+from .utils import (
+    API_KEY_COOKIE_NAME,
+    CSRF_COOKIE_NAME,
+    get_api_access_manager,
+    get_authenticators,
+    get_default_login_data,
+    get_resource_access_manager,
+    record_timing,
+)
 
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.INFO)
 
 logging.basicConfig(level=logging.WARNING)
-logging.getLogger("bluesky_queueserver").setLevel("DEBUG")
+# logging.getLogger("bluesky_queueserver").setLevel("DEBUG")
+logging.getLogger(__name__).setLevel("DEBUG")
 
 SAFE_METHODS = {"GET", "HEAD", "OPTIONS", "TRACE"}
 SENSITIVE_COOKIES = {
@@ -140,10 +139,16 @@ def build_app(authentication=None, api_access=None, resource_access=None, server
     app.include_router(core_api.router)
 
     # Include custom routers
+    router_names = []
     router_names_str = os.getenv("QSERVER_HTTP_CUSTOM_ROUTERS", None)
-    if router_names_str:
+    if "custom_routers" in server_settings["server_configuration"]:
+        router_names = server_settings["server_configuration"]["custom_routers"]
+        logger.info("Custom routers are specified in the config file: %s", router_names)
+    elif router_names_str:
         router_names = re.split(":|,", router_names_str)
-        logger.info("Custom routers to include (env. variable): %s", pprint.pformat(router_names))
+        logger.info("Custom routers are specified in the environment variable: %s", router_names)
+
+    if router_names:
         routers_already_included = set()
         for rn in router_names:
             if rn and (rn not in routers_already_included):
@@ -165,7 +170,6 @@ def build_app(authentication=None, api_access=None, resource_access=None, server
     authentication_router.include_router(base_authentication_router)
 
     if authentication.get("providers", []):
-
         # For the OpenAPI schema, inject a OAuth2PasswordBearer URL.
         first_provider = authentication["providers"][0]["provider"]
         oauth2_scheme.model.flows.password.tokenUrl = f"/api/auth/provider/{first_provider}/token"
@@ -197,7 +201,6 @@ def build_app(authentication=None, api_access=None, resource_access=None, server
 
     @app.on_event("startup")
     async def startup_event():
-
         # Validate the single-user API key.
         settings = app.dependency_overrides[get_settings]()
         single_user_api_key = settings.single_user_api_key
@@ -221,31 +224,21 @@ def build_app(authentication=None, api_access=None, resource_access=None, server
         background_tasks = []
         for authenticator in authenticators:
             background_tasks.extend(getattr(authenticator, "background_tasks", []))
+        background_tasks.extend(getattr(api_access_manager, "background_tasks", []))
         for task in background_tasks or []:
             asyncio_task = asyncio.create_task(task())
             app.state.tasks.append(asyncio_task)
 
-        app.state.allow_origins.extend(settings.allow_origins)
-        app.add_middleware(
-            CORSMiddleware,
-            allow_origins=app.state.allow_origins,
-            allow_credentials=True,
-            allow_methods=["*"],
-            allow_headers=["*"],
-        )
-
         if settings.database_uri is not None:
             from sqlalchemy import create_engine
 
             # from sqlalchemy.orm import sessionmaker
-
             from .database import orm
-            from .database.core import (
+            from .database.core import (  # make_admin_by_identity,
                 REQUIRED_REVISION,
                 UninitializedDatabase,
                 check_database,
                 initialize_database,
-                # make_admin_by_identity,
             )
 
             connect_args = {}
@@ -284,15 +277,6 @@ def build_app(authentication=None, api_access=None, resource_access=None, server
 
             app.state.tasks.append(asyncio.create_task(purge_expired_sessions_and_api_keys()))
 
-        # Read private key from the environment variable, then check if the CLI parameter exists
-        zmq_public_key = os.environ.get("QSERVER_ZMQ_PUBLIC_KEY", None)
-        zmq_public_key = zmq_public_key if zmq_public_key else None  # Case of ""
-        if zmq_public_key is not None:
-            try:
-                validate_zmq_key(zmq_public_key)
-            except Exception as ex:
-                raise ValueError("ZMQ public key is improperly formatted: %s", str(ex))
-
         # TODO: implement nicer exit with error reporting in case of failure
         zmq_control_addr = os.getenv("QSERVER_ZMQ_CONTROL_ADDRESS", None)
         if zmq_control_addr is None:
@@ -323,6 +307,20 @@ def build_app(authentication=None, api_access=None, resource_access=None, server
                     "QSERVER_ZMQ_INFO_ADDRESS to pass address of 0MQ information socket to HTTP Server."
                 )
 
+        # Check if ZMQ setting were specified in config file. Overrid the parameters from EVs.
+        zmq_control_addr = server_settings["qserver_zmq_configuration"].get("control_address", zmq_control_addr)
+        zmq_info_addr = server_settings["qserver_zmq_configuration"].get("info_address", zmq_info_addr)
+
+        # Read public key from the environment variable or config file.
+        zmq_public_key = os.environ.get("QSERVER_ZMQ_PUBLIC_KEY", None)
+        zmq_public_key = zmq_public_key if zmq_public_key else None  # Case of ""
+        zmq_public_key = server_settings["qserver_zmq_configuration"].get("public_key", zmq_public_key)
+        if zmq_public_key is not None:
+            try:
+                validate_zmq_key(zmq_public_key)
+            except Exception as ex:
+                raise ValueError(f"ZMQ public key is improperly formatted: {ex}")
+
         logger.info(
             f"Connecting to RE Manager: \nControl 0MQ socket address: {zmq_control_addr}\n"
             f"Information 0MQ socket address: {zmq_info_addr}"
@@ -355,10 +353,15 @@ def build_app(authentication=None, api_access=None, resource_access=None, server
             )
         module_names_str = module_names_str or os.getenv("QSERVER_CUSTOM_MODULE", None)
 
-        if module_names_str:
+        module_names = []
+        if "custom_modules" in server_settings["server_configuration"]:
+            module_names = server_settings["server_configuration"]["custom_modules"]
+            logger.info("Custom modules from config file: %s", pprint.pformat(module_names))
+        elif module_names_str:
             module_names = re.split(":|,", module_names_str)
-            logger.info("Custom modules to import (env. variable): %s", pprint.pformat(module_names))
+            logger.info("Custom modules from environment variable: %s", pprint.pformat(module_names))
 
+        if module_names:
             # Import all listed custom modules
             custom_code_modules = []
             for name in module_names:
@@ -511,4 +514,17 @@ def build_app(authentication=None, api_access=None, resource_access=None, server
     app.dependency_overrides[get_resource_access_manager] = override_get_resource_access_manager
     app.dependency_overrides[get_settings] = override_get_settings
 
+    def add_custom_middleware():
+        settings = app.dependency_overrides[get_settings]()
+        app.state.allow_origins.extend(settings.allow_origins)
+        app.add_middleware(
+            CORSMiddleware,
+            allow_origins=app.state.allow_origins,
+            allow_credentials=True,
+            allow_methods=["*"],
+            allow_headers=["*"],
+        )
+
+    add_custom_middleware()
+
     return app
